@@ -59,10 +59,12 @@ EMPTY = "<empty>"
         'extra == "a" and extra != "b"',
         'extra != "a" and extra == "b"',
         'extra != "a" and extra != "b"',
+        'extra == "a" and extra == "b" and extra != "c" and extra != "d"',
         'extra == "a" or extra == "b"',
         'extra == "a" or extra != "b"',
         'extra != "a" or extra == "b"',
         'extra != "a" or extra != "b"',
+        'extra == "a" or extra == "b" or extra != "c" or extra != "d"',
         # String comparison markers
         '"tegra" in platform_release',
         '"tegra" not in platform_release',
@@ -169,11 +171,24 @@ def test_single_marker_intersect() -> None:
     assert str(intersection) == 'python_version >= "3.4" and python_version < "3.6"'
 
 
-def test_single_marker_intersect_compacts_constraints() -> None:
-    m = parse_marker('python_version < "3.6"')
+@pytest.mark.parametrize(
+    ("marker1", "marker2", "expected"),
+    [
+        ('python_version < "3.6"', 'python_version < "3.4"', 'python_version < "3.4"'),
+        (
+            'python_version >= "3.6"',
+            'python_version < "3.7"',
+            'python_version == "3.6"',
+        ),
+    ],
+)
+def test_single_marker_intersect_is_single_marker(
+    marker1: str, marker2: str, expected: str
+) -> None:
+    m = parse_marker(marker1)
 
-    intersection = m.intersect(parse_marker('python_version < "3.4"'))
-    assert str(intersection) == 'python_version < "3.4"'
+    intersection = m.intersect(parse_marker(marker2))
+    assert str(intersection) == expected
 
 
 def test_single_marker_intersect_with_multi() -> None:
@@ -250,6 +265,98 @@ def test_single_marker_not_in_python_intersection() -> None:
         ('extra == "a"', 'extra != "b"', 'extra == "a" and extra != "b"'),
         ('extra != "a"', 'extra == "b"', 'extra != "a" and extra == "b"'),
         ('extra != "a"', 'extra != "b"', 'extra != "a" and extra != "b"'),
+        # AtomicMultiMarker
+        (
+            'extra == "a" and extra == "b"',
+            'extra == "c"',
+            'extra == "a" and extra == "b" and extra == "c"',
+        ),
+        (
+            'extra != "a" and extra != "b"',
+            'extra != "c"',
+            'extra != "a" and extra != "b" and extra != "c"',
+        ),
+        (
+            'extra != "a" and extra != "b"',
+            'extra == "c"',
+            'extra != "a" and extra != "b" and extra == "c"',
+        ),
+        (
+            'extra == "a" and extra == "b"',
+            'extra != "c"',
+            'extra == "a" and extra == "b" and extra != "c"',
+        ),
+        (
+            'extra == "a" and extra == "b"',
+            'extra == "a" and extra == "b"',
+            'extra == "a" and extra == "b"',
+        ),
+        (
+            'extra == "a" and extra == "b"',
+            'extra == "b" and extra == "a"',
+            'extra == "a" and extra == "b"',
+        ),
+        (
+            'extra == "a" and extra == "b"',
+            'extra == "c" and extra != "d"',
+            'extra == "a" and extra == "b" and extra == "c" and extra != "d"',
+        ),
+        ('extra != "a" and extra != "b"', 'extra == "a"', "<empty>"),
+        ('extra != "a" and extra == "b"', 'extra == "a" and extra == "c"', "<empty>"),
+        (
+            'extra != "a" and extra != "b"',
+            'extra != "a"',
+            'extra != "a" and extra != "b"',
+        ),
+        (
+            'extra == "a" and extra != "b"',
+            'extra == "a"',
+            'extra == "a" and extra != "b"',
+        ),
+        # AtomicMarkerUnion
+        (
+            'extra == "a" or extra == "b"',
+            'extra == "c"',
+            '(extra == "a" or extra == "b") and extra == "c"',
+        ),
+        (
+            'extra == "a" or extra == "b"',
+            'extra != "c"',
+            '(extra == "a" or extra == "b") and extra != "c"',
+        ),
+        ('extra == "a" or extra == "b"', 'extra == "a"', 'extra == "a"'),
+        ('extra != "a" or extra == "b"', 'extra != "a"', 'extra != "a"'),
+        (
+            'extra == "a" or extra == "b"',
+            'extra != "a"',
+            'extra == "b" and extra != "a"',
+        ),
+        (
+            'extra == "a" or extra == "b"',
+            'extra == "a" or extra == "b"',
+            'extra == "a" or extra == "b"',
+        ),
+        (
+            'extra == "a" or extra == "b"',
+            'extra == "b" or extra == "a"',
+            'extra == "a" or extra == "b"',
+        ),
+        (
+            'extra == "a" or extra == "b"',
+            'extra == "a" or extra != "c"',
+            '(extra == "a" or extra == "b") and (extra == "a" or extra != "c")',
+        ),
+        # AtomicMultiMarker and AtomicMarkerUnion
+        (
+            'extra != "a" and extra != "b"',
+            'extra == "a" or extra == "b"',
+            "<empty>",
+        ),
+        (
+            'extra != "a" and extra != "b"',
+            'extra == "a" or extra == "c"',
+            'extra == "c" and extra != "a" and extra != "b"',
+        ),
     ],
 )
 def test_single_marker_intersect_extras(
@@ -300,6 +407,11 @@ def test_single_marker_union_is_any() -> None:
             'python_version > "3.6"',
             'python_version != "3.6"',
         ),
+        (
+            'python_version == "3.6"',
+            'python_version >= "3.7"',
+            'python_version >= "3.6"',
+        ),
     ],
 )
 def test_single_marker_union_is_single_marker(
@@ -443,6 +555,104 @@ def test_single_marker_union_with_inverse() -> None:
         ('extra == "a"', 'extra != "b"', 'extra == "a" or extra != "b"'),
         ('extra != "a"', 'extra == "b"', 'extra != "a" or extra == "b"'),
         ('extra != "a"', 'extra != "b"', 'extra != "a" or extra != "b"'),
+        # AtomicMultiMarker
+        (
+            'extra == "a" and extra == "b"',
+            'extra == "c"',
+            'extra == "a" and extra == "b" or extra == "c"',
+        ),
+        (
+            'extra != "a" and extra != "b"',
+            'extra != "c"',
+            'extra != "a" and extra != "b" or extra != "c"',
+        ),
+        (
+            'extra != "a" and extra != "b"',
+            'extra == "c"',
+            'extra != "a" and extra != "b" or extra == "c"',
+        ),
+        (
+            'extra == "a" and extra == "b"',
+            'extra != "c"',
+            'extra == "a" and extra == "b" or extra != "c"',
+        ),
+        (
+            'extra == "a" and extra == "b"',
+            'extra == "a" and extra == "b"',
+            'extra == "a" and extra == "b"',
+        ),
+        (
+            'extra == "a" and extra == "b"',
+            'extra == "c" and extra != "d"',
+            'extra == "a" and extra == "b" or extra == "c" and extra != "d"',
+        ),
+        (
+            'extra != "a" and extra != "b"',
+            'extra == "a"',
+            'extra != "b" or extra == "a"',
+        ),
+        (
+            'extra != "a" and extra == "b"',
+            'extra == "a" and extra == "c"',
+            'extra != "a" and extra == "b" or extra == "a" and extra == "c"',
+        ),
+        (
+            'extra != "a" and extra != "b"',
+            'extra != "a"',
+            'extra != "a"',
+        ),
+        (
+            'extra == "a" and extra != "b"',
+            'extra == "a"',
+            'extra == "a"',
+        ),
+        # AtomicMarkerUnion
+        (
+            'extra == "a" or extra == "b"',
+            'extra == "c"',
+            'extra == "a" or extra == "b" or extra == "c"',
+        ),
+        (
+            'extra == "a" or extra == "b"',
+            'extra != "c"',
+            'extra == "a" or extra == "b" or extra != "c"',
+        ),
+        (
+            'extra == "a" or extra == "b"',
+            'extra == "a"',
+            'extra == "a" or extra == "b"',
+        ),
+        (
+            'extra != "a" or extra == "b"',
+            'extra != "a"',
+            'extra != "a" or extra == "b"',
+        ),
+        (
+            'extra == "a" or extra == "b"',
+            'extra != "a"',
+            "",
+        ),
+        (
+            'extra == "a" or extra == "b"',
+            'extra == "a" or extra == "b"',
+            'extra == "a" or extra == "b"',
+        ),
+        (
+            'extra == "a" or extra == "b"',
+            'extra == "a" or extra != "c"',
+            'extra == "a" or extra == "b" or extra != "c"',
+        ),
+        # AtomicMultiMarker and AtomicMarkerUnion
+        (
+            'extra != "a" and extra != "b"',
+            'extra == "a" or extra == "b"',
+            'extra != "a" and extra != "b" or extra == "a" or extra == "b"',
+        ),
+        (
+            'extra != "a" and extra != "b"',
+            'extra == "a" or extra == "c"',
+            'extra != "a" and extra != "b" or extra == "a" or extra == "c"',
+        ),
     ],
 )
 def test_single_marker_union_extras(marker1: str, marker2: str, expected: str) -> None:
@@ -610,35 +820,65 @@ def test_multi_marker_union_multi_is_multi(
         (
             'python_version >= "3.6" and python_full_version < "3.6.2"',
             'python_version >= "3.6" and python_version < "3.7"',
-            'python_version >= "3.6" and python_version < "3.7"',
+            'python_version == "3.6"',
+        ),
+        (
+            'python_version >= "3.6" and python_full_version < "3.7.2"',
+            'python_version >= "3.6" and python_version < "3.8"',
+            'python_version >= "3.6" and python_version < "3.8"',
         ),
         (
             'python_version > "3.6" and python_full_version < "3.6.2"',
             'python_version > "3.6" and python_version < "3.7"',
             'python_version > "3.6" and python_version < "3.7"',
         ),
+        (
+            'python_version > "3.6" and python_full_version < "3.7.2"',
+            'python_version > "3.6" and python_version < "3.8"',
+            'python_version > "3.6" and python_version < "3.8"',
+        ),
         # Ranges meet exactly
         (
             'python_version >= "3.6" and python_full_version < "3.6.2"',
             'python_full_version >= "3.6.2" and python_version < "3.7"',
-            'python_version >= "3.6" and python_version < "3.7"',
+            'python_version == "3.6"',
+        ),
+        (
+            'python_version >= "3.6" and python_full_version < "3.7.2"',
+            'python_full_version >= "3.6.2" and python_version < "3.8"',
+            'python_version >= "3.6" and python_version < "3.8"',
         ),
         (
             'python_version >= "3.6" and python_full_version <= "3.6.2"',
             'python_full_version > "3.6.2" and python_version < "3.7"',
-            'python_version >= "3.6" and python_version < "3.7"',
+            'python_version == "3.6"',
+        ),
+        (
+            'python_version >= "3.6" and python_full_version <= "3.7.2"',
+            'python_full_version > "3.6.2" and python_version < "3.8"',
+            'python_version >= "3.6" and python_version < "3.8"',
         ),
         # Ranges overlap
         (
             'python_version >= "3.6" and python_full_version <= "3.6.8"',
             'python_full_version >= "3.6.2" and python_version < "3.7"',
-            'python_version >= "3.6" and python_version < "3.7"',
+            'python_version == "3.6"',
+        ),
+        (
+            'python_version >= "3.6" and python_full_version <= "3.7.8"',
+            'python_full_version >= "3.6.2" and python_version < "3.8"',
+            'python_version >= "3.6" and python_version < "3.8"',
         ),
         # Ranges with same end.
         (
             'python_version >= "3.6" and python_version < "3.7"',
             'python_full_version >= "3.6.2" and python_version < "3.7"',
-            'python_version >= "3.6" and python_version < "3.7"',
+            'python_version == "3.6"',
+        ),
+        (
+            'python_version >= "3.6" and python_version < "3.8"',
+            'python_full_version >= "3.6.2" and python_version < "3.8"',
+            'python_version >= "3.6" and python_version < "3.8"',
         ),
         (
             'python_version >= "3.6" and python_version <= "3.7"',
@@ -748,9 +988,8 @@ def test_marker_union_intersect_single_marker() -> None:
 
     intersection = m.intersect(parse_marker('implementation_name == "cpython"'))
     assert (
-        str(intersection)
-        == 'sys_platform == "darwin" and implementation_name == "cpython" '
-        'or python_version < "3.4" and implementation_name == "cpython"'
+        str(intersection) == '(sys_platform == "darwin" or python_version < "3.4")'
+        ' and implementation_name == "cpython"'
     )
 
 
@@ -776,11 +1015,8 @@ def test_marker_union_intersect_marker_union() -> None:
         parse_marker('implementation_name == "cpython" or os_name == "Windows"')
     )
     assert (
-        str(intersection)
-        == 'sys_platform == "darwin" and implementation_name == "cpython" '
-        'or sys_platform == "darwin" and os_name == "Windows" or '
-        'python_version < "3.4" and implementation_name == "cpython" or '
-        'python_version < "3.4" and os_name == "Windows"'
+        str(intersection) == '(sys_platform == "darwin" or python_version < "3.4") and '
+        '(implementation_name == "cpython" or os_name == "Windows")'
     )
 
 
@@ -796,8 +1032,7 @@ def test_marker_union_intersect_marker_union_drops_unnecessary_markers() -> None
 
     intersection = m.intersect(m2)
     expected = (
-        'python_version >= "2.7" and python_version < "2.8" '
-        'or python_version >= "3.4" and python_version < "4.0"'
+        'python_version == "2.7" or python_version >= "3.4" and python_version < "4.0"'
     )
     assert str(intersection) == expected
 
@@ -808,18 +1043,16 @@ def test_marker_union_intersect_multi_marker() -> None:
 
     # Intersection isn't _quite_ symmetrical.
     expected1 = (
-        'sys_platform == "darwin" and implementation_name == "cpython" and os_name =='
-        ' "Windows" or python_version < "3.4" and implementation_name == "cpython" and'
-        ' os_name == "Windows"'
+        '(sys_platform == "darwin" or python_version < "3.4")'
+        ' and implementation_name == "cpython" and os_name == "Windows"'
     )
 
     intersection = m1.intersect(m2)
     assert str(intersection) == expected1
 
     expected2 = (
-        'implementation_name == "cpython" and os_name == "Windows" and sys_platform'
-        ' == "darwin" or implementation_name == "cpython" and os_name == "Windows"'
-        ' and python_version < "3.4"'
+        'implementation_name == "cpython" and os_name == "Windows"'
+        ' and (sys_platform == "darwin" or python_version < "3.4")'
     )
 
     intersection = m2.intersect(m1)
@@ -1165,6 +1398,15 @@ def test_without_extras(marker: str, expected: str) -> None:
             "extra",
             'python_version >= "3.6"',
         ),
+        (
+            (
+                'python_version >= "2.7" and (python_version < "2.8"'
+                ' or python_version >= "3.7") and python_version < "3.8"'
+                ' and extra == "foo"'
+            ),
+            "extra",
+            ('python_version == "2.7" or python_version == "3.7"'),
+        ),
     ],
 )
 def test_exclude(marker: str, excluded: str, expected: str) -> None:
@@ -1241,7 +1483,12 @@ def test_only(marker: str, only: list[str], expected: str) -> None:
         ("<empty>", "~3.8", "<empty>"),
         ('sys_platform == "linux"', "~3.8", 'sys_platform == "linux"'),
         ('python_version >= "3.8"', "~3.8", ""),
-        ('python_version > "3.8"', "~3.8", 'python_version > "3.8"'),
+        ('python_version == "3.8"', ">=3.8.7,<3.9.0", ""),
+        ('python_version == "3.8" or python_version >= "3.9"', ">=3.8.0,<4.0.0", ""),
+        ('python_version == "3.8" or python_version >= "3.9"', ">=3.8.7,<4.0.0", ""),
+        ('python_version > "3.7"', "~3.8", ""),
+        ('python_version > "3.8"', "~3.8", "<empty>"),
+        ('python_version >= "3.8"', "~3.8", ""),
         ('python_version >= "3.9"', "~3.8", "<empty>"),
         ('python_full_version >= "3.8.0"', "~3.8", ""),
         ('python_full_version >= "3.8.1"', "~3.8", 'python_full_version >= "3.8.1"'),
@@ -1305,6 +1552,16 @@ def test_only(marker: str, only: list[str], expected: str) -> None:
             "~3.7 || ~3.9",
             'sys_platform == "linux" or sys_platform == "win32"',
         ),
+        (
+            'python_version == "3.8" or sys_platform == "linux" or python_version >= "3.9"',
+            ">=3.8.0,<4.0.0",
+            'sys_platform == "linux"',
+        ),
+        (
+            'python_version == "3.8" or sys_platform == "linux" or python_version >= "3.9"',
+            ">=3.8.7,<4.0.0",
+            'sys_platform == "linux"',
+        ),
     ],
 )
 def test_reduce_by_python_constraint(
@@ -1839,8 +2096,8 @@ def test_complex_union() -> None:
     ]
     assert (
         str(union(*markers))
-        == 'platform_system == "Darwin" and platform_machine == "arm64"'
-        ' and python_version >= "3.6" or python_version >= "3.10"'
+        == 'python_version >= "3.6" and platform_system == "Darwin"'
+        ' and platform_machine == "arm64" or python_version >= "3.10"'
     )
 
 
@@ -1872,8 +2129,8 @@ def test_complex_intersection() -> None:
     ]
     assert (
         str(dnf(intersection(*markers).invert()))
-        == 'platform_system == "Darwin" and platform_machine == "arm64"'
-        ' and python_version >= "3.6" or python_version >= "3.10"'
+        == 'python_version >= "3.6" and platform_system == "Darwin"'
+        ' and platform_machine == "arm64" or python_version >= "3.10"'
     )
 
 
@@ -1919,14 +2176,14 @@ def test_intersection_avoids_combinatorial_explosion() -> None:
     )
     assert (
         str(m1.intersect(m2))
-        == 'python_full_version >= "3.11.0" and python_full_version < "4.0.0"'
+        == 'python_version >= "3.11" and python_full_version < "4.0.0"'
         ' and (platform_machine == "aarch64" or platform_machine == "ppc64le"'
         ' or platform_machine == "x86_64" or platform_machine == "amd64"'
         ' or platform_machine == "AMD64" or platform_machine == "win32"'
         ' or platform_machine == "WIN32")'
     )
     assert (
-        str(m2.intersect(m1)) == 'python_full_version >= "3.11.0"'
+        str(m2.intersect(m1)) == 'python_version >= "3.11"'
         ' and (platform_machine == "aarch64" or platform_machine == "ppc64le"'
         ' or platform_machine == "x86_64" or platform_machine == "amd64"'
         ' or platform_machine == "AMD64" or platform_machine == "win32"'
@@ -1934,6 +2191,30 @@ def test_intersection_avoids_combinatorial_explosion() -> None:
     )
 
 
+def test_intersection_no_endless_recursion() -> None:
+    m1 = parse_marker(
+        '(python_version < "3.9" or extra != "bigquery" and extra != "parquet"'
+        ' and extra != "motherduck" and extra != "athena" and extra != "synapse"'
+        ' and extra != "clickhouse" and extra != "dremio" and extra != "lancedb"'
+        ' and extra != "deltalake" and extra != "pyiceberg"'
+        ' and python_version < "3.13") and extra != "postgres" and extra != "redshift"'
+        ' and extra != "postgis"'
+    )
+    m2 = parse_marker(
+        'python_version > "3.12" and python_version < "3.13" or extra != "databricks"'
+    )
+    expected = (
+        '(python_version < "3.9" or extra != "bigquery" and extra != "parquet"'
+        ' and extra != "motherduck" and extra != "athena" and extra != "synapse"'
+        ' and extra != "clickhouse" and extra != "dremio" and extra != "lancedb"'
+        ' and extra != "deltalake" and extra != "pyiceberg")'
+        ' and python_version < "3.13" and extra != "postgres" and extra != "redshift"'
+        ' and extra != "postgis" and (python_version > "3.12"'
+        ' and python_version < "3.13" or extra != "databricks")'
+    )
+    assert str(m1.intersect(m2)) == expected
+
+
 @pytest.mark.parametrize(
     "python_version, python_full_version, "
     "expected_intersection_version, expected_union_version",
@@ -2009,20 +2290,20 @@ def test_intersection_avoids_combinatorial_explosion() -> None:
         ('== "3.6"', '> "3.5.2"', '== "3.6"', '> "3.5.2"'),
         ('== "3.6"', '>= "3.5.2"', '== "3.6"', '>= "3.5.2"'),
         ('== "3.6"', '!= "3.5.2"', '== "3.6"', '!= "3.5.2"'),
-        ('== "3.6"', '< "3.6.0"', EMPTY, '< "3.7.0"'),
-        ('== "3.6"', '<= "3.6.0"', '== "3.6.0"', '< "3.7.0"'),
-        ('== "3.6"', '> "3.6.0"', None, '>= "3.6.0"'),
-        ('== "3.6"', '>= "3.6.0"', '== "3.6"', '>= "3.6.0"'),
+        ('== "3.6"', '< "3.6.0"', EMPTY, '< "3.7"'),
+        ('== "3.6"', '<= "3.6.0"', '== "3.6.0"', '< "3.7"'),
+        ('== "3.6"', '> "3.6.0"', None, '>= "3.6"'),
+        ('== "3.6"', '>= "3.6.0"', '== "3.6"', '>= "3.6"'),
         ('== "3.6"', '!= "3.6.0"', None, ""),
-        ('== "3.6"', '< "3.6.1"', None, '< "3.7.0"'),
-        ('== "3.6"', '<= "3.6.1"', None, '< "3.7.0"'),
-        ('== "3.6"', '> "3.6.1"', None, '>= "3.6.0"'),
-        ('== "3.6"', '>= "3.6.1"', None, '>= "3.6.0"'),
+        ('== "3.6"', '< "3.6.1"', None, '< "3.7"'),
+        ('== "3.6"', '<= "3.6.1"', None, '< "3.7"'),
+        ('== "3.6"', '> "3.6.1"', None, '>= "3.6"'),
+        ('== "3.6"', '>= "3.6.1"', None, '>= "3.6"'),
         ('== "3.6"', '!= "3.6.1"', None, ""),
-        ('== "3.6"', '< "3.7.0"', '== "3.6"', '< "3.7.0"'),
+        ('== "3.6"', '< "3.7.0"', '== "3.6"', '< "3.7"'),
         ('== "3.6"', '<= "3.7.0"', '== "3.6"', '<= "3.7.0"'),
         ('== "3.6"', '> "3.7.0"', EMPTY, None),
-        ('== "3.6"', '>= "3.7.0"', EMPTY, '>= "3.6.0"'),
+        ('== "3.6"', '>= "3.7.0"', EMPTY, '>= "3.6"'),
         ('== "3.6"', '!= "3.7.0"', '== "3.6"', '!= "3.7.0"'),
         ('== "3.6"', '<= "3.7.1"', '== "3.6"', '<= "3.7.1"'),
         ('== "3.6"', '< "3.7.1"', '== "3.6"', '< "3.7.1"'),
@@ -2036,20 +2317,20 @@ def test_intersection_avoids_combinatorial_explosion() -> None:
         ('!= "3.6"', '> "3.5.2"', None, ""),
         ('!= "3.6"', '>= "3.5.2"', None, ""),
         ('!= "3.6"', '!= "3.5.2"', None, ""),
-        ('!= "3.6"', '< "3.6.0"', '< "3.6.0"', '!= "3.6"'),
-        ('!= "3.6"', '<= "3.6.0"', '< "3.6.0"', None),
-        ('!= "3.6"', '> "3.6.0"', '>= "3.7.0"', '!= "3.6.0"'),
-        ('!= "3.6"', '>= "3.6.0"', '>= "3.7.0"', ""),
+        ('!= "3.6"', '< "3.6.0"', '< "3.6"', '!= "3.6"'),
+        ('!= "3.6"', '<= "3.6.0"', '< "3.6"', None),
+        ('!= "3.6"', '> "3.6.0"', '>= "3.7"', '!= "3.6.0"'),
+        ('!= "3.6"', '>= "3.6.0"', '>= "3.7"', ""),
         ('!= "3.6"', '!= "3.6.0"', '!= "3.6"', '!= "3.6.0"'),
-        ('!= "3.6"', '< "3.6.1"', '< "3.6.0"', None),
-        ('!= "3.6"', '<= "3.6.1"', '< "3.6.0"', None),
-        ('!= "3.6"', '> "3.6.1"', '>= "3.7.0"', None),
-        ('!= "3.6"', '>= "3.6.1"', '>= "3.7.0"', None),
+        ('!= "3.6"', '< "3.6.1"', '< "3.6"', None),
+        ('!= "3.6"', '<= "3.6.1"', '< "3.6"', None),
+        ('!= "3.6"', '> "3.6.1"', '>= "3.7"', None),
+        ('!= "3.6"', '>= "3.6.1"', '>= "3.7"', None),
         ('!= "3.6"', '!= "3.6.1"', '!= "3.6"', '!= "3.6.1"'),
-        ('!= "3.6"', '< "3.7.0"', '< "3.6.0"', ""),
+        ('!= "3.6"', '< "3.7.0"', '< "3.6"', ""),
         ('!= "3.6"', '<= "3.7.0"', None, ""),
         ('!= "3.6"', '> "3.7.0"', '> "3.7.0"', '!= "3.6"'),
-        ('!= "3.6"', '>= "3.7.0"', '>= "3.7.0"', '!= "3.6"'),
+        ('!= "3.6"', '>= "3.7.0"', '>= "3.7"', '!= "3.6"'),
         ('!= "3.6"', '!= "3.7.0"', None, ""),
         ('!= "3.6"', '<= "3.7.1"', None, ""),
         ('!= "3.6"', '< "3.7.1"', None, ""),
